--------------------------------------------------------------------------------
--
-- Copyright (C) 2011
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- JIVE (Joint Institute for VLBI in Europe) <http://www.jive.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--------------------------------------------------------------------------------

-- This testbench tests the different type of DDR controllers: 
--
--                        - aphy_4g_800
--                        - aphy_4g_1066
--                        - uphy_4g_800_master
--                        - uphy_4g_1066_master
--                        - uphy_4g_800_slave
--                        - uphy_4g_1066_slave
--
-- The DUT can be selected, using the c_phy and c_mts constants. 
--
-- Testbench is selftesting: 
--
-- > run -all 
--
-- Known issues: The memory model used for the aphy_4g_1066 simulation
--               gives an error at the end of the write cyclus.
-- 

LIBRARY IEEE, tech_ddr_lib, common_lib, dp_lib, diagnostics_lib;                   
USE IEEE.STD_LOGIC_1164.ALL;    
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.tb_common_mem_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
USE tech_ddr_lib.tech_ddr_pkg.ALL;
USE work.ddr3_pkg.ALL;

ENTITY tb_ddr3 IS  
END ENTITY tb_ddr3;

ARCHITECTURE str of tb_ddr3 IS

  CONSTANT c_phy              : NATURAL      := 1;     -- 0: ALTMEMPHY  1: UNIPHY_MASTER 2: UNIPHY_SLAVE 
  CONSTANT c_mts              : NATURAL      := 800;  -- Available options: 800 and 1066
  
  CONSTANT c_ctlr_ref_clk_per : TIME         := 5 ns; --200 MHz
  CONSTANT c_ddr              : t_c_ddr3_phy := c_ddr3_phy_4g;

  CONSTANT c_data_w           : NATURAL      := 256; --32;
 
  SIGNAL ctlr_ref_clk         : STD_LOGIC    := '0';
  SIGNAL ctlr_rst             : STD_LOGIC    := '1';
  SIGNAL tb_end               : STD_LOGIC    := '0';
  SIGNAL ctlr_gen_clk         : STD_LOGIC;
  SIGNAL ctlr_gen_rst         : STD_LOGIC;

  SIGNAL ctlr_rdy             : STD_LOGIC;
  SIGNAL ctlr_init_done       : STD_LOGIC;

  SIGNAL dvr_start_addr       : t_ddr3_addr; 
  SIGNAL dvr_end_addr         : t_ddr3_addr;

  SIGNAL dvr_en               : STD_LOGIC;
  SIGNAL dvr_wr_not_rd        : STD_LOGIC;
  SIGNAL dvr_done             : STD_LOGIC;

  SIGNAL wr_siso              : t_dp_siso;
  SIGNAL wr_sosi              : t_dp_sosi;

  SIGNAL wr_siso_pre_fifo     : t_dp_siso;
  SIGNAL wr_sosi_pre_fifo     : t_dp_sosi;

  SIGNAL rd_siso              : t_dp_siso;
  SIGNAL rd_sosi              : t_dp_sosi;

  SIGNAL src_diag_en          : STD_LOGIC;
  SIGNAL src_val_cnt          : STD_LOGIC_VECTOR(31 DOWNTO 0);

  SIGNAL snk_diag_en          : STD_LOGIC;
  SIGNAL snk_diag_res         : STD_LOGIC;
  SIGNAL snk_diag_res_val     : STD_LOGIC;
  SIGNAL snk_val_cnt          : STD_LOGIC_VECTOR(31 DOWNTO 0);

  SIGNAL phy_in               : t_tech_ddr3_phy_in;
  SIGNAL phy_io               : t_tech_ddr3_phy_io;
  SIGNAL phy_ou               : t_tech_ddr3_phy_ou;
  
  SIGNAL ras_n                : STD_LOGIC_VECTOR(0 DOWNTO 0);
  SIGNAL cas_n                : STD_LOGIC_VECTOR(0 DOWNTO 0);
  SIGNAL we_n                 : STD_LOGIC_VECTOR(0 DOWNTO 0);
  
  SIGNAL flush_ena            : STD_LOGIC;
  
BEGIN
 
  ctlr_ref_clk   <= NOT(ctlr_ref_clk) OR tb_end AFTER c_ctlr_ref_clk_per/2; 
  ctlr_rst       <= '0' AFTER 100 ns;

  dvr_start_addr <= c_ddr3_addr_lo;
  dvr_end_addr   <= c_ddr3_addr_hi_sim;
  
  flush_ena      <= '0';

  p_stimuli : PROCESS
  BEGIN
    tb_end        <= '0';
    dvr_en        <= '0';
    src_diag_en   <= '0';
    dvr_wr_not_rd <= '0';
    snk_diag_en   <= '0';

    WAIT UNTIL ctlr_init_done = '1';   
    FOR i IN 0 TO 1 LOOP
      WAIT UNTIL rising_edge(ctlr_gen_clk); -- Give the driver FSM a cycle to go into idle mode
    END LOOP;
    
    -- START WRITE
    src_diag_en   <= '1';
    dvr_wr_not_rd <= '1';
    dvr_en        <= '1';

    WAIT UNTIL rising_edge(ctlr_gen_clk);

    dvr_en        <= '0'; 
    
    -- WRITE DONE  
    WAIT UNTIL dvr_done = '1';

    src_diag_en   <= '0';

    -- START READ
    snk_diag_en   <= '1';
    dvr_wr_not_rd <= '0';
    dvr_en        <= '1';

    WAIT UNTIL rising_edge(ctlr_gen_clk);

    dvr_en        <= '0'; 

    -- READ DONE
    WAIT UNTIL dvr_done = '1';    

    WAIT FOR 2 us; -- 'Done' means all requests are posted. Wait for the last read data to arrive.
  
    ASSERT snk_diag_res_val = '1' REPORT "[ERROR] DIAG_RES INVALID!" SEVERITY FAILURE;
    ASSERT snk_diag_res = '0' REPORT "[ERROR] NON-ZERO DIAG_RES!" SEVERITY FAILURE;
    ASSERT FALSE REPORT "[OK] Test passed." SEVERITY NOTE;
    tb_end        <= '1';

    WAIT;
  END PROCESS;

  u_diagnostics: ENTITY diagnostics_lib.diagnostics 
  GENERIC MAP (
    g_dat_w             => c_data_w,
    g_nof_streams       => 1
     ) 
  PORT MAP (
    rst                 => ctlr_gen_rst,
    clk                 => ctlr_gen_clk,

    snk_out_arr(0)      => rd_siso,
    snk_in_arr(0)       => rd_sosi,
    snk_diag_en(0)      => snk_diag_en,
    snk_diag_md(0)      => '1',
    snk_diag_res(0)     => snk_diag_res,
    snk_diag_res_val(0) => snk_diag_res_val,
    snk_val_cnt(0)      => snk_val_cnt,

    src_out_arr(0)      => wr_sosi,
    src_in_arr(0)       => wr_siso,
    src_diag_en(0)      => src_diag_en,
    src_diag_md(0)      => '1',
    src_val_cnt(0)      => src_val_cnt
  );

  
  gen_uphy_4g_model : IF c_phy > 0 GENERATE
    u_4gb_ddr3_model : COMPONENT alt_mem_if_ddr3_mem_model_top_ddr3_mem_if_dm_pins_en_mem_if_dqsn_en
	  GENERIC MAP (
	  	MEM_IF_ADDR_WIDTH            => 15,
	  	MEM_IF_ROW_ADDR_WIDTH        => 15,
	  	MEM_IF_COL_ADDR_WIDTH        => 10,
	  	MEM_IF_CS_PER_RANK           => 1,
	  	MEM_IF_CONTROL_WIDTH         => 1,
	  	MEM_IF_DQS_WIDTH             => 8,
	  	MEM_IF_CS_WIDTH              => 2,
	  	MEM_IF_BANKADDR_WIDTH        => 3,
	  	MEM_IF_DQ_WIDTH              => 64,
	  	MEM_IF_CK_WIDTH              => 2,
	  	MEM_IF_CLK_EN_WIDTH          => 2,
	  	DEVICE_WIDTH                 => 1,
	  	MEM_TRCD                     => 6,
	  	MEM_TRTP                     => 3,
	  	MEM_DQS_TO_CLK_CAPTURE_DELAY => 100,
	  	MEM_CLK_TO_DQS_CAPTURE_DELAY => 100000,
	  	MEM_IF_ODT_WIDTH             => 2,
	  	MEM_MIRROR_ADDRESSING_DEC    => 0,
	  	MEM_REGDIMM_ENABLED          => false,
	  	DEVICE_DEPTH                 => 1,
	  	MEM_GUARANTEED_WRITE_INIT    => false,
	  	MEM_VERBOSE                  => true,
	  	MEM_INIT_EN                  => false,
	  	MEM_INIT_FILE                => "",
	  	DAT_DATA_WIDTH               => 32
	  )
	  PORT MAP (
	  	mem_a       => phy_ou.a(c_ddr.a_w-1 DOWNTO 0),        -- memory.mem_a
	  	mem_ba      => phy_ou.ba,      --       .mem_ba
	  	mem_ck      => phy_ou.ck,      --       .mem_ck
	  	mem_ck_n    => phy_ou.ck_n,    --       .mem_ck_n
	  	mem_cke     => phy_ou.cke(c_ddr.cs_w-1 DOWNTO 0),     --       .mem_cke
	  	mem_cs_n    => phy_ou.cs_n(c_ddr.cs_w-1 DOWNTO 0),    --       .mem_cs_n
	  	mem_dm      => phy_ou.dm,      --       .mem_dm
	  	mem_ras_n   => ras_n,          --       .mem_ras_n
	  	mem_cas_n   => cas_n,          --       .mem_cas_n
	  	mem_we_n    => we_n,           --       .mem_we_n
	  	mem_reset_n => phy_ou.reset_n, --       .mem_reset_n
	  	mem_dq      => phy_io.dq,      --       .mem_dq
	  	mem_dqs     => phy_io.dqs,     --       .mem_dqs
	  	mem_dqs_n   => phy_io.dqs_n,   --       .mem_dqs_n
	  	mem_odt     => phy_ou.odt      --       .mem_odt
	  );               
	  
	  ras_n(0) <= phy_ou.ras_n;
	  cas_n(0) <= phy_ou.cas_n;
	  we_n(0)  <= phy_ou.we_n; 
	END GENERATE;
  
  u_ddr3_module: ENTITY work.ddr3
  GENERIC MAP(
    g_phy              => c_phy,
    g_mts              => c_mts,
    g_ddr              => c_ddr,
    g_wr_data_w        => c_data_w,
    g_rd_data_w        => c_data_w
  )                      
  PORT MAP (                  
    ctlr_ref_clk       => ctlr_ref_clk,
    ctlr_rst           => ctlr_rst,
                                     
    ctlr_gen_clk       => ctlr_gen_clk,
    ctlr_gen_rst       => ctlr_gen_rst,      

    ctlr_init_done     => ctlr_init_done,
    ctlr_rdy           => ctlr_rdy,

    dvr_start_addr     => dvr_start_addr,
    dvr_end_addr       => dvr_end_addr,
    dvr_en             => dvr_en,
    dvr_wr_not_rd      => dvr_wr_not_rd,
    dvr_done           => dvr_done,
 
    wr_clk             => ctlr_gen_clk,
    wr_rst             => ctlr_gen_rst,

    wr_sosi            => wr_sosi, 
    wr_siso            => wr_siso,
    
    flush_ena          => flush_ena,
  
    rd_sosi            => rd_sosi,
    rd_siso            => rd_siso,

    rd_clk             => ctlr_gen_clk,
    rd_rst             => ctlr_gen_rst,

    phy_ou             => phy_ou,
    phy_io             => phy_io,
    phy_in             => phy_in
  );
 
END ARCHITECTURE str;


