-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Provide DP tx and DP rx interface to 10GbE IO.
-- Description
--
-- . Block diagram:
--                                         - XAUI
--                       ST                - 10GBASE-R
--   dp_rst -->           |                    |
--   dp_clk -->           |                    |
--                        |                    |
--    snk_in  --> fifo ---|--> tech_eth_10g ---|-->  i/o pins
--    src_out <--      <--|---              <--|---
--                        |        ^ |         |
--                                 | |
--                          _______|_v______
--       tr_ref_clk_644 --> |              |
--       tr_ref_clk_312 --> | tech_eth_10g |
--       tr_ref_clk_156 --> | clocks       |
--       tr_ref_rst_156 --> |______________|
--
-- . If the average DP data rate > 10GbE data rate then the g_tx_fifo_fill
--   can be small, otherwise:
--
--      g_tx_fifo_fill > (1-(DP data rate)/156.25MHz) * largest packet size
--
--   to avoid that the packet transmission will get a gap that will abort it.
--   The average DP data rate depends on dp_clk and on the DP data valid.
--  

LIBRARY IEEE, common_lib, dp_lib, diag_lib, technology_lib, tech_mac_10g_lib, tech_eth_10g_lib, tr_xaui_lib;
USE IEEE.std_logic_1164.ALL;
USE IEEE.numeric_std.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_network_layers_pkg.ALL;
USE common_lib.common_interface_layers_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE technology_lib.technology_select_pkg.ALL;
USE technology_lib.technology_pkg.ALL;
USE tech_mac_10g_lib.tech_mac_10g_component_pkg.ALL;

ENTITY tr_10GbE IS
  GENERIC (
    g_technology             : NATURAL := c_tech_select_default;
    g_sim                    : BOOLEAN;
    g_sim_level              : NATURAL := 1; -- 0 = use IP; 1 = use fast serdes model
    g_nof_macs               : NATURAL;
    g_use_mdio               : BOOLEAN := FALSE;
    g_mdio_epcs_dis          : BOOLEAN := FALSE; -- TRUE disables EPCS on init; e.g. to target a 10GbE card in PC that does not support it 
    g_tx_fifo_fill           : NATURAL := 10;    -- Release tx packet only when sufficiently data is available, 
    g_tx_fifo_size           : NATURAL := 256;   -- 2 * 32b * 256 = 2 M9K (DP interface has 64b data, so at least 2 M9K needed)
    g_rx_fifo_size           : NATURAL := 256;   -- 2 * 32b * 256 = 2 M9K (DP interface has 64b data, so at least 2 M9K needed)
    g_word_alignment_padding : BOOLEAN := FALSE
  );
  PORT (
    -- Transceiver PLL reference clock
    tr_ref_clk_644   : IN  STD_LOGIC := '0';   -- 644.531250 MHz for 10GBASE-R
    tr_ref_clk_312   : IN  STD_LOGIC := '0';   -- 312.5      MHz for 10GBASE-R
    tr_ref_clk_156   : IN  STD_LOGIC := '0';   -- 156.25     MHz for 10GBASE-R or for XAUI
    tr_ref_rst_156   : IN  STD_LOGIC := '0';   --                for 10GBASE-R or for XAUI
    
    -- Calibration & reconfig clock
    cal_rec_clk         : IN  STD_LOGIC := '0';   -- for XAUI;
    
    -- MM interface
    mm_rst              : IN  STD_LOGIC;
    mm_clk              : IN  STD_LOGIC;
    
    reg_mac_mosi        : IN  t_mem_mosi := c_mem_mosi_rst;
    reg_mac_miso        : OUT t_mem_miso;

    xaui_mosi           : IN  t_mem_mosi := c_mem_mosi_rst;
    xaui_miso           : OUT t_mem_miso;
    
    mdio_mosi_arr       : IN  t_mem_mosi_arr(g_nof_macs-1 DOWNTO 0) := (OTHERS=>c_mem_mosi_rst);
    mdio_miso_arr       : OUT t_mem_miso_arr(g_nof_macs-1 DOWNTO 0);

    -- DP interface
    dp_rst              : IN  STD_LOGIC := '0';
    dp_clk              : IN  STD_LOGIC := '0';

    snk_out_arr         : OUT t_dp_siso_arr(g_nof_macs-1 DOWNTO 0);
    snk_in_arr          : IN  t_dp_sosi_arr(g_nof_macs-1 DOWNTO 0) := (OTHERS=>c_dp_sosi_rst);

    src_in_arr          : IN  t_dp_siso_arr(g_nof_macs-1 DOWNTO 0) := (OTHERS=>c_dp_siso_rdy);
    src_out_arr         : OUT t_dp_sosi_arr(g_nof_macs-1 DOWNTO 0);
    
    -- Serial XAUI IO
    xaui_tx_arr         : OUT t_xaui_arr(g_nof_macs-1 DOWNTO 0);
    xaui_rx_arr         : IN  t_xaui_arr(g_nof_macs-1 DOWNTO 0) := (OTHERS=>(OTHERS=>'0'));

    -- Serial IO
    serial_tx_arr       : OUT STD_LOGIC_VECTOR(g_nof_macs-1 downto 0);
    serial_rx_arr       : IN  STD_LOGIC_VECTOR(g_nof_macs-1 downto 0) := (OTHERS=>'0');
  
    -- MDIO interface
    mdio_rst            : OUT STD_LOGIC;
    mdio_mdc_arr        : OUT STD_LOGIC_VECTOR(g_nof_macs-1 DOWNTO 0);       
    mdio_mdat_in_arr    : IN  STD_LOGIC_VECTOR(g_nof_macs-1 DOWNTO 0) := (OTHERS=>'0');
    mdio_mdat_oen_arr   : OUT STD_LOGIC_VECTOR(g_nof_macs-1 DOWNTO 0)
  );
END tr_10GbE;


ARCHITECTURE str OF tr_10GbE IS

  SIGNAL tx_rst_arr_out                 : STD_LOGIC_VECTOR(g_nof_macs-1 DOWNTO 0);
  SIGNAL rx_clk_arr_out                 : STD_LOGIC_VECTOR(g_nof_macs-1 DOWNTO 0);
  SIGNAL rx_rst_arr_out                 : STD_LOGIC_VECTOR(g_nof_macs-1 DOWNTO 0);
  
  SIGNAL eth_ref_clk_644                : STD_LOGIC;
  SIGNAL eth_ref_clk_312                : STD_LOGIC;
  SIGNAL eth_ref_clk_156                : STD_LOGIC;
  SIGNAL eth_ref_rst_156                : STD_LOGIC;
  
  SIGNAL eth_tx_clk_arr                 : STD_LOGIC_VECTOR(g_nof_macs-1 DOWNTO 0);
  SIGNAL eth_tx_rst_arr                 : STD_LOGIC_VECTOR(g_nof_macs-1 DOWNTO 0);
  SIGNAL eth_rx_clk_arr                 : STD_LOGIC_VECTOR(g_nof_macs-1 DOWNTO 0);
  SIGNAL eth_rx_rst_arr                 : STD_LOGIC_VECTOR(g_nof_macs-1 DOWNTO 0);

  SIGNAL dp_fifo_dc_tx_src_out_arr      : t_dp_sosi_arr(g_nof_macs-1 DOWNTO 0); 
  SIGNAL dp_fifo_dc_tx_src_in_arr       : t_dp_siso_arr(g_nof_macs-1 DOWNTO 0);

  SIGNAL dp_fifo_fill_tx_src_out_arr    : t_dp_sosi_arr(g_nof_macs-1 DOWNTO 0); 
  SIGNAL dp_fifo_fill_tx_src_in_arr     : t_dp_siso_arr(g_nof_macs-1 DOWNTO 0);

  SIGNAL mac_10g_src_out_arr            : t_dp_sosi_arr(g_nof_macs-1 DOWNTO 0); 
  SIGNAL mac_10g_src_in_arr             : t_dp_siso_arr(g_nof_macs-1 DOWNTO 0);

  SIGNAL dp_fifo_dc_rx_src_out_arr      : t_dp_sosi_arr(g_nof_macs-1 DOWNTO 0); 
  SIGNAL dp_fifo_dc_rx_src_in_arr       : t_dp_siso_arr(g_nof_macs-1 DOWNTO 0);

BEGIN

  ---------------------------------------------------------------------------------------
  -- Clocks and reset
  ---------------------------------------------------------------------------------------

  -- Apply the clocks from top level down such that they have their rising_edge() aligned without any delta-delay
  u_tech_eth_10g_clocks : ENTITY tech_eth_10g_lib.tech_eth_10g_clocks
  GENERIC MAP (
    g_technology     => g_technology,
    g_nof_channels   => g_nof_macs
  )
  PORT MAP (
    -- Input clocks
    -- . Reference
    tr_ref_clk_644    => tr_ref_clk_644,
    tr_ref_clk_312    => tr_ref_clk_312,
    tr_ref_clk_156    => tr_ref_clk_156,
    tr_ref_rst_156    => tr_ref_rst_156,
    
    -- . XAUI
    tx_rst_arr        => tx_rst_arr_out,
    rx_clk_arr        => rx_clk_arr_out,
    rx_rst_arr        => rx_rst_arr_out,
    
    -- Output clocks
    -- . Reference
    eth_ref_clk_644   => eth_ref_clk_644,
    eth_ref_clk_312   => eth_ref_clk_312,
    eth_ref_clk_156   => eth_ref_clk_156,
    eth_ref_rst_156   => eth_ref_rst_156,
    
    -- . Data
    eth_tx_clk_arr    => eth_tx_clk_arr,
    eth_tx_rst_arr    => eth_tx_rst_arr,
    
    eth_rx_clk_arr    => eth_rx_clk_arr,
    eth_rx_rst_arr    => eth_rx_rst_arr
  );  
  
  ---------------------------------------------------------------------------------------
  -- TX: FIFO: dp_clk -> tx_clk and with fill level so we can deliver packets to the MAC fast enough
  ---------------------------------------------------------------------------------------
  gen_dp_fifo_fill_dc : FOR i IN 0 TO g_nof_macs-1 GENERATE
    u_dp_fifo_fill_dc : ENTITY dp_lib.dp_fifo_fill_dc
    GENERIC MAP (
      g_data_w      => c_xgmii_data_w,
      g_empty_w     => c_tech_mac_10g_empty_w,
      g_use_empty   => TRUE,
      g_fifo_fill   => g_tx_fifo_fill,
      g_fifo_size   => g_tx_fifo_size
    )
    PORT MAP (
      wr_rst      => dp_rst,
      wr_clk      => dp_clk,
      rd_rst      => eth_tx_rst_arr(i),
      rd_clk      => eth_tx_clk_arr(i),

      snk_out     => snk_out_arr(i),
      snk_in      => snk_in_arr(i),

      src_in      => dp_fifo_fill_tx_src_in_arr(i),
      src_out     => dp_fifo_fill_tx_src_out_arr(i)
    );
  END GENERATE;
  
  
  ---------------------------------------------------------------------------------------
  -- ETH MAC + PHY
  ---------------------------------------------------------------------------------------

  u_tech_eth_10g : ENTITY tech_eth_10g_lib.tech_eth_10g
  GENERIC MAP (
    g_technology          => g_technology,
    g_sim                 => g_sim,
    g_sim_level           => g_sim_level,  -- 0 = use IP; 1 = use fast serdes model
    g_nof_channels        => g_nof_macs,
    g_link_status_check   => "11",
    g_pre_header_padding  => g_word_alignment_padding
  )
  PORT MAP (
    -- Transceiver PLL reference clock
    tr_ref_clk_644   => eth_ref_clk_644,   -- 644.531250 MHz for 10GBASE-R
    tr_ref_clk_312   => eth_ref_clk_312,   -- 312.5      MHz for 10GBASE-R
    tr_ref_clk_156   => eth_ref_clk_156,   -- 156.25     MHz for 10GBASE-R or for XAUI
    tr_ref_rst_156   => eth_ref_rst_156,   --                for 10GBASE-R or for XAUI
    
    -- Calibration & reconfig clock
    cal_rec_clk      => cal_rec_clk,      -- for XAUI;
    
    -- XAUI clocks
    tx_clk_arr_in    => eth_tx_clk_arr,   -- 156.25 MHz, externally connect tr_ref_clk_156 to tx_clk_arr or connect rx_clk_arr to tx_clk_arr
    tx_rst_arr_out   => tx_rst_arr_out,
    rx_clk_arr_out   => rx_clk_arr_out,
    rx_clk_arr_in    => eth_rx_clk_arr,   -- externally connect to rx_clk_arr_out to avoid clock delta-delay
    rx_rst_arr_out   => rx_rst_arr_out,
    
    -- MM
    mm_clk           => mm_clk,
    mm_rst           => mm_rst,
    
    mac_mosi         => reg_mac_mosi,    -- MAG_10G (CSR), aggregated for all g_nof_channels
    mac_miso         => reg_mac_miso, 
    
    xaui_mosi        => xaui_mosi,       -- XAUI control
    xaui_miso        => xaui_miso,
    
    -- ST
    tx_snk_in_arr    => dp_fifo_fill_tx_src_out_arr,      -- 64 bit data @ 156 MHz
    tx_snk_out_arr   => dp_fifo_fill_tx_src_in_arr ,
    
    rx_src_out_arr   => mac_10g_src_out_arr,              -- 64 bit data @ 156 MHz
    rx_src_in_arr    => mac_10g_src_in_arr,
    
    -- PHY serial IO
    -- . 10GBASE-R (single lane)
    serial_tx_arr    => serial_tx_arr,
    serial_rx_arr    => serial_rx_arr,
      
    -- . XAUI (four lanes)
    xaui_tx_arr      => xaui_tx_arr,
    xaui_rx_arr      => xaui_rx_arr
  );

  ---------------------------------------------------------------------------
  -- MDIO
  ---------------------------------------------------------------------------
  gen_mdio: IF g_use_mdio = TRUE GENERATE
    u_tr_xaui_mdio : ENTITY tr_xaui_lib.tr_xaui_mdio
    GENERIC MAP (
      g_sim           => g_sim,
      g_nof_xaui      => g_nof_macs,
      g_mdio_epcs_dis => g_mdio_epcs_dis
    )
    PORT MAP (
      -- Transceiver PLL reference clock   
      tr_clk            => eth_ref_clk_156,
      tr_rst            => eth_ref_rst_156,
    
      -- MM clock for register of optional MDIO master
      mm_clk            => mm_clk,
      mm_rst            => mm_rst,
    
      -- MDIO master = mm slave
      mdio_mosi_arr     => mdio_mosi_arr,
      mdio_miso_arr     => mdio_miso_arr,
    
      -- MDIO External clock and serial data.
      mdio_rst          => mdio_rst,
      mdio_mdc_arr      => mdio_mdc_arr,
      mdio_mdat_in_arr  => mdio_mdat_in_arr,
      mdio_mdat_oen_arr => mdio_mdat_oen_arr
    );
  END GENERATE;

  ---------------------------------------------------------------------------------------
  -- RX FIFO: rx_clk -> dp_clk
  ---------------------------------------------------------------------------------------
  gen_dp_fifo_dc_rx : FOR i IN g_nof_macs-1 DOWNTO 0 GENERATE
    u_dp_fifo_dc_rx : ENTITY dp_lib.dp_fifo_dc
    GENERIC MAP (
      g_data_w      => c_xgmii_data_w,
      g_empty_w     => c_tech_mac_10g_empty_w,
      g_use_empty   => TRUE,
      g_fifo_size   => g_rx_fifo_size
    )
    PORT MAP (
      wr_rst      => eth_rx_rst_arr(i),
      wr_clk      => eth_rx_clk_arr(i),
      rd_rst      => dp_rst,
      rd_clk      => dp_clk,
  
      snk_out     => mac_10g_src_in_arr(i),
      snk_in      => mac_10g_src_out_arr(i),
  
      src_in      => dp_fifo_dc_rx_src_in_arr(i), 
      src_out     => dp_fifo_dc_rx_src_out_arr(i)
    );   
  END GENERATE;

  -----------------------------------------------------------------------------
  -- RX XON frame control
  -----------------------------------------------------------------------------
  gen_dp_xonoff : FOR i IN g_nof_macs-1 DOWNTO 0 GENERATE
    u_dp_xonoff : ENTITY dp_lib.dp_xonoff
    PORT MAP (
      rst      => dp_rst,
      clk      => dp_clk,
           
      in_siso  => dp_fifo_dc_rx_src_in_arr(i),      
      in_sosi  => dp_fifo_dc_rx_src_out_arr(i),
      
      out_siso => src_in_arr(i),
      out_sosi => src_out_arr(i)    
    );   
  END GENERATE;

END str;
