-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Testbench for tech_mac_10g the 10G Ethernet IP technology wrapper.
-- Description:
--   The tb is self checking based on:
--   . proc_tech_mac_10g_rx_packet() for expected header and data type
--   . tx_pkt_cnt=rx_pkt_cnt > 0 must be true at the tb_end.
--
--   The tb finishes using severity failure because the simulation can not
--   finish by stopping the clocks and applying the resets due to that the IP
--   keeps on toggling internally.
-- Usage:
--   > as 10
--   > run -all

LIBRARY IEEE, technology_lib, tech_pll_lib, tech_mac_10g_lib, common_lib, dp_lib;
USE IEEE.std_logic_1164.ALL;
USE common_lib.common_pkg.ALL;
USE common_lib.common_mem_pkg.ALL;
USE common_lib.common_interface_layers_pkg.ALL;
USE common_lib.common_network_layers_pkg.ALL;
USE common_lib.common_network_total_header_pkg.ALL;
USE common_lib.tb_common_pkg.ALL;
USE dp_lib.dp_stream_pkg.ALL;
USE technology_lib.technology_pkg.ALL;
USE technology_lib.technology_select_pkg.ALL;
USE tech_mac_10g_lib.tech_mac_10g_component_pkg.ALL;
USE tech_mac_10g_lib.tb_tech_mac_10g_pkg.ALL;
USE tech_pll_lib.tech_pll_component_pkg.ALL;

ENTITY tb_tr_10GbE IS
  -- Test bench control parameters
  GENERIC (
    g_technology              : NATURAL := c_tech_select_default;
    g_tb_end                  : BOOLEAN := TRUE;   -- when TRUE then tb_end ends this simulation, else a higher multi-testbench will end the simulation
    g_no_dut                  : BOOLEAN := FALSE;  -- default FALSE to verify the DUT, else use TRUE to verify the tb itself without the DUT
    g_dp_clk_period           : TIME := 5 ns;      -- must be ~< 9000/(9000-c_tx_fifo_fill) * g_ref_clk_156_period
    g_sim_level               : NATURAL := 0;      -- 0 = use IP; 1 = use fast serdes model
    g_nof_channels            : NATURAL := 1;
    g_ref_clk_644_period      : TIME := tech_pll_clk_644_period;  -- for 10GBASE-R
    g_ref_clk_156_period      : TIME := 6.4 ns;                   -- for XAUI
    g_data_type               : NATURAL := c_tb_tech_mac_10g_data_type_symbols;
    g_verify_link_recovery    : BOOLEAN := TRUE;
    g_link_status_check       : STD_LOGIC_VECTOR(c_tech_mac_10g_link_status_w-1 DOWNTO 0) := "11"
  );
  PORT (
    tb_end : OUT STD_LOGIC
  );
END tb_tr_10GbE;


ARCHITECTURE tb OF tb_tr_10GbE IS

  CONSTANT c_sim                : BOOLEAN := TRUE;  -- tr_xaui_align_dly has time delay ~ 1 sec, so not suitable for simulation
  
  CONSTANT cal_clk_period       : TIME := 25 ns;    --  40 MHz
  
  CONSTANT phy_delay            : TIME := sel_a_b(g_sim_level=0, 1 ns, 0 ns);
  
  CONSTANT c_tx_fifo_fill       : NATURAL := 100;
   
  CONSTANT c_pkt_length_arr1    : t_nat_natural_arr := array_init(0, 50, 1) & (1472, 1473) & 9000;  -- frame longer than 1518-46 = 1472 is received with rx_sosi.err = 8
                                                                                                    -- jumbo frame is 9018-46 = 8972
  CONSTANT c_pkt_length_arr2    : t_nat_natural_arr := array_init(46, 10, 139) & 1472;
  CONSTANT c_pkt_length_arr     : t_nat_natural_arr := c_pkt_length_arr1 & c_pkt_length_arr2;
  CONSTANT c_nof_pkt1           : NATURAL := c_pkt_length_arr1'LENGTH;
  CONSTANT c_nof_pkt2           : NATURAL := c_pkt_length_arr2'LENGTH;
  CONSTANT c_nof_pkt            : NATURAL := sel_a_b(g_verify_link_recovery, c_nof_pkt1 + c_nof_pkt2, c_nof_pkt1);

  CONSTANT c_dst_mac            : STD_LOGIC_VECTOR(c_network_eth_mac_slv'RANGE) := X"10FA01020300";
  CONSTANT c_src_mac            : STD_LOGIC_VECTOR(c_network_eth_mac_slv'RANGE) := X"123456789ABC";  -- = 12-34-56-78-9A-BC
  CONSTANT c_src_mac_tx         : STD_LOGIC_VECTOR(c_network_eth_mac_slv'RANGE) := c_src_mac;
  --CONSTANT c_src_mac_tx         : STD_LOGIC_VECTOR(c_network_eth_mac_slv'RANGE) := X"100056789ABC";  -- = 10-00-56-78-9A-BC
  CONSTANT c_ethertype          : STD_LOGIC_VECTOR(c_network_eth_type_slv'RANGE) := X"10FA";
  CONSTANT c_etherlen           : STD_LOGIC_VECTOR(c_network_eth_type_slv'RANGE) := "0000000000010000";

  -- Packet headers
  CONSTANT c_eth_header_ethertype : t_network_eth_header := (c_dst_mac, c_src_mac_tx, c_ethertype);
  CONSTANT c_eth_header_etherlen  : t_network_eth_header := (c_dst_mac, c_src_mac_tx, c_etherlen);
  
  SIGNAL total_header      : t_network_total_header := c_network_total_header_ones;  -- default fill all fields with value 1
  
  -- Clocks and reset
  SIGNAL tx_end_arr        : STD_LOGIC_VECTOR(g_nof_channels-1 DOWNTO 0);
  SIGNAL tx_end            : STD_LOGIC;
  SIGNAL rx_end            : STD_LOGIC;
  SIGNAL cal_clk           : STD_LOGIC := '1';  -- calibration clock
  SIGNAL mm_clk            : STD_LOGIC;         -- memory-mapped bus clock
  SIGNAL mm_rst            : STD_LOGIC;         -- reset synchronous with mm_clk
  SIGNAL dp_clk            : STD_LOGIC := '1';  -- data path clock
  SIGNAL dp_rst            : STD_LOGIC;         -- reset synchronous with dp_clk
  
  -- External reference clocks
  SIGNAL tr_ref_clk_644    : STD_LOGIC := '1';  -- 10GBASE-R
  SIGNAL tr_ref_clk_312    : STD_LOGIC;         -- 10GBASE-R
  SIGNAL tr_ref_clk_156    : STD_LOGIC := '1';  -- 10GBASE-R or XAUI
  SIGNAL tr_ref_rst_156    : STD_LOGIC;         -- 10GBASE-R or XAUI
    
  -- MAC 10G control interface
  SIGNAL mm_init           : STD_LOGIC;
  SIGNAL mac_mosi_wrdata   : STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0);  -- for channel 0, 32 bit
  SIGNAL mac_mosi          : t_mem_mosi;
  SIGNAL mac_miso          : t_mem_miso;
  SIGNAL mac_miso_rdval    : STD_LOGIC;                              -- for channel 0
  SIGNAL mac_miso_rddata   : STD_LOGIC_VECTOR(c_word_w-1 DOWNTO 0);  -- for channel 0, 32 bit
  
  -- MAC 10G transmit interface
  SIGNAL tx_en             : STD_LOGIC := '1';
  SIGNAL tx_siso_arr       : t_dp_siso_arr(g_nof_channels-1 DOWNTO 0);
  SIGNAL tx_sosi_arr       : t_dp_sosi_arr(g_nof_channels-1 DOWNTO 0);
  SIGNAL tx_sosi_data      : STD_LOGIC_VECTOR(c_tech_mac_10g_data_w-1 DOWNTO 0);  -- for channel 0, 64 bit
  
  -- MAC 10G receive interface
  SIGNAL rx_siso_arr       : t_dp_siso_arr(g_nof_channels-1 DOWNTO 0);
  SIGNAL rx_sosi_arr       : t_dp_sosi_arr(g_nof_channels-1 DOWNTO 0);
  SIGNAL rx_sosi_data      : STD_LOGIC_VECTOR(c_tech_mac_10g_data_w-1 DOWNTO 0);  -- for channel 0, 64 bit

  -- PHY XAUI serial I/O
  SIGNAL xaui_tx_arr       : t_xaui_arr(g_nof_channels-1 DOWNTO 0);
  SIGNAL xaui_rx_arr       : t_xaui_arr(g_nof_channels-1 DOWNTO 0) := (OTHERS=>(OTHERS=>'0'));

  -- PHY 10GBASE-R serial IO
  SIGNAL serial_tx_arr     : STD_LOGIC_VECTOR(g_nof_channels-1 downto 0);
  SIGNAL serial_rx_arr     : STD_LOGIC_VECTOR(g_nof_channels-1 downto 0) := (OTHERS=>'0');

  -- Model a serial link fault
  SIGNAL link_fault_arr    : STD_LOGIC_VECTOR(g_nof_channels-1 downto 0);
  
  -- Verification
  SIGNAL tx_pkt_cnt_arr    : t_natural_arr(g_nof_channels-1 downto 0) := (OTHERS=>0);
  SIGNAL rx_pkt_cnt_arr    : t_natural_arr(g_nof_channels-1 downto 0) := (OTHERS=>0);
  SIGNAL rx_toggle_arr     : STD_LOGIC_VECTOR(g_nof_channels-1 downto 0) := (OTHERS=>'0');  -- toggle after every received packet
  
BEGIN

  cal_clk <= NOT cal_clk AFTER cal_clk_period/2;  -- Calibration clock
  
  dp_clk  <= NOT dp_clk  AFTER g_dp_clk_period/2;   -- DP clock
  dp_rst  <= '1', '0' AFTER g_dp_clk_period*10;

  -- debug signals to ease monitoring in wave window
  mac_mosi_wrdata <= mac_mosi.wrdata(c_word_w-1 DOWNTO 0);
  mac_miso_rddata <= mac_miso.rddata(c_word_w-1 DOWNTO 0);
  mac_miso_rdval <= '1' WHEN mac_mosi.rd='1' AND mac_miso.waitrequest='0' ELSE '0';  -- c_rd_latency = 1
  
  tx_sosi_data <= tx_sosi_arr(0).data(c_tech_mac_10g_data_w-1 DOWNTO 0);
  rx_sosi_data <= rx_sosi_arr(0).data(c_tech_mac_10g_data_w-1 DOWNTO 0);
  
  -- Use signal to leave unused fields 'X'
  total_header.eth <= c_eth_header_ethertype;

  -- Generate reference clocks
  gen_ref_clocks_xaui : IF g_technology=c_tech_stratixiv GENERATE
    tr_ref_clk_644 <= 'X';
    tr_ref_clk_312 <= 'X';
    tr_ref_clk_156 <= NOT tr_ref_clk_156 AFTER g_ref_clk_156_period/2;
    tr_ref_rst_156 <= '1', '0' AFTER g_ref_clk_156_period*5;
  END GENERATE;
    
  gen_ref_clocks_10gbase_r : IF g_technology=c_tech_arria10 GENERATE
    tr_ref_clk_644 <= NOT tr_ref_clk_644 AFTER g_ref_clk_644_period/2;
  
    pll : ENTITY tech_pll_lib.tech_pll_xgmii_mac_clocks
    GENERIC MAP (
      g_technology => g_technology
    )
    PORT MAP (
      refclk_644 => tr_ref_clk_644,
      rst_in     => mm_rst,
      clk_156    => tr_ref_clk_156,
      clk_312    => tr_ref_clk_312,
      rst_156    => tr_ref_rst_156,
      rst_312    => OPEN
    );
  END GENERATE;
  
  -- Setup all MACs in series
  u_mm_setup : ENTITY tech_mac_10g_lib.tb_tech_mac_10g_setup
  GENERIC MAP (
    g_technology    => g_technology,
    g_nof_macs      => g_nof_channels,
    g_src_mac       => c_src_mac
  )
  PORT MAP (
    tb_end    => rx_end,
    mm_clk    => mm_clk,
    mm_rst    => mm_rst,
    mm_init   => mm_init,
    mac_mosi  => mac_mosi,
    mac_miso  => mac_miso
  );  
  
  gen_transmitter : FOR I IN 0 TO g_nof_channels-1 GENERATE
    -- Packet transmitter
    u_transmitter : ENTITY tech_mac_10g_lib.tb_tech_mac_10g_transmitter
    GENERIC MAP (
      g_data_type            => g_data_type,
      g_pkt_length_arr1      => c_pkt_length_arr1,
      g_pkt_length_arr2      => c_pkt_length_arr2,
      g_verify_link_recovery => g_verify_link_recovery
    )
    PORT MAP (
      mm_init        => mm_init,
      total_header   => total_header,
      tx_clk         => dp_clk,
      tx_siso        => tx_siso_arr(I),
      tx_sosi        => tx_sosi_arr(I),
      link_fault     => link_fault_arr(I),
      tx_end         => tx_end_arr(I)
    );
  END GENERATE;

  no_dut : IF g_no_dut=TRUE GENERATE
    rx_sosi_arr <= tx_sosi_arr;
    tx_siso_arr <= rx_siso_arr;
  END GENERATE;

  gen_dut : IF g_no_dut=FALSE GENERATE
    u_tr_10GbE : ENTITY work.tr_10GbE
    GENERIC MAP (
      g_technology             => g_technology,
      g_sim                    => c_sim,
      g_sim_level              => g_sim_level, -- 0 = use IP; 1 = use fast serdes model
      g_nof_macs               => g_nof_channels,
      g_use_mdio               => TRUE,
      g_mdio_epcs_dis          => TRUE,  -- TRUE disables Enhanced PCS on init; e.g. to target a 10GbE card in PC that does not support it 
      g_tx_fifo_fill           => c_tx_fifo_fill,
      g_tx_fifo_size           => 256,
      g_word_alignment_padding => TRUE
    )
    PORT MAP (
      -- Transceiver PLL reference clock
      tr_ref_clk_644      => tr_ref_clk_644,   -- 644.531250 MHz for 10GBASE-R
      tr_ref_clk_312      => tr_ref_clk_312,   -- 312.5      MHz for 10GBASE-R
      tr_ref_clk_156      => tr_ref_clk_156,   -- 156.25     MHz for 10GBASE-R or for XAUI
      tr_ref_rst_156      => tr_ref_rst_156,   --                for 10GBASE-R or for XAUI

      -- Calibration & reconfig clock
      cal_rec_clk         => cal_clk,
      
      -- MM interface
      mm_rst              => mm_rst,
      mm_clk              => mm_clk,
      
      reg_mac_mosi        => mac_mosi,
      reg_mac_miso        => mac_miso,
  
      xaui_mosi           => c_mem_mosi_rst,
      xaui_miso           => OPEN,
      
      mdio_mosi_arr       => (OTHERS=>c_mem_mosi_rst),
      mdio_miso_arr       => OPEN,
  
      -- DP interface
      dp_rst              => dp_rst,
      dp_clk              => dp_clk,
  
      snk_out_arr         => tx_siso_arr,
      snk_in_arr          => tx_sosi_arr,
  
      src_in_arr          => rx_siso_arr,
      src_out_arr         => rx_sosi_arr,
  
      -- Serial XAUI interface
      xaui_tx_arr         => xaui_tx_arr,
      xaui_rx_arr         => xaui_rx_arr,
  
      -- Serial IO
      serial_tx_arr       => serial_tx_arr,
      serial_rx_arr       => serial_rx_arr,
    
      -- MDIO interface
      mdio_rst            => OPEN,
      mdio_mdc_arr        => OPEN,
      mdio_mdat_in_arr    => (OTHERS=>'0'),
      mdio_mdat_oen_arr   => OPEN
    );
  END GENERATE;
  
  gen_link_connect : FOR I IN 0 TO g_nof_channels-1 GENERATE
    u_link_connect : ENTITY tech_mac_10g_lib.tb_tech_mac_10g_link_connect
    GENERIC MAP (
      g_loopback    => TRUE,
      g_link_delay  => phy_delay
    )
    PORT MAP (
      link_fault   => link_fault_arr(I), -- when '1' then forces rx_serial_arr(0)='0'
      
      -- 10GBASE-R serial layer connect
      serial_tx    => serial_tx_arr(I),
      serial_rx    => serial_rx_arr(I),  -- connects to delayed tx_serial when g_loopback=TRUE
      
      -- XAUI serial layer connect
      xaui_tx      => xaui_tx_arr(I),
      xaui_rx      => xaui_rx_arr(I)     -- connects to delayed xaui_tx when g_loopback=TRUE
    );
  END GENERATE;
  
  gen_receiver : FOR I IN 0 TO g_nof_channels-1 GENERATE
    u_receiver : ENTITY tech_mac_10g_lib.tb_tech_mac_10_receiver
    GENERIC MAP (
      g_data_type  => g_data_type
    )
    PORT MAP (
      mm_init        => mm_init,
      total_header   => total_header,
      rx_clk         => dp_clk,
      rx_sosi        => rx_sosi_arr(I),
      rx_siso        => rx_siso_arr(I),
      rx_toggle      => rx_toggle_arr(I)
    );
  END GENERATE;
  
  gen_verify_rx_at_eop : FOR I IN 0 TO g_nof_channels-1 GENERATE
    u_verify_rx_at_eop : ENTITY tech_mac_10g_lib.tb_tech_mac_10_verify_rx_at_eop
    GENERIC MAP (
      g_no_padding     => g_no_dut,
      g_pkt_length_arr => c_pkt_length_arr
    )
    PORT MAP (
      tx_clk      => dp_clk,
      tx_sosi     => tx_sosi_arr(I),
      rx_clk      => dp_clk,
      rx_sosi     => rx_sosi_arr(I)
    );  
  END GENERATE;
  
  gen_verify_rx_pkt_cnt : FOR I IN 0 TO g_nof_channels-1 GENERATE
    u_verify_rx_pkt_cnt : ENTITY tech_mac_10g_lib.tb_tech_mac_10g_verify_rx_pkt_cnt
    GENERIC MAP (
      g_nof_pkt   => c_nof_pkt
    )
    PORT MAP (
      tx_clk      => dp_clk,
      tx_sosi     => tx_sosi_arr(I),
      rx_clk      => dp_clk,
      rx_sosi     => rx_sosi_arr(I),
      tx_pkt_cnt  => tx_pkt_cnt_arr(I),
      rx_pkt_cnt  => rx_pkt_cnt_arr(I),
      rx_end      => rx_end
    );
  END GENERATE;

  -- Stop the simulation
  tx_end <= andv(tx_end_arr);
  
  u_simulation_end : ENTITY tech_mac_10g_lib.tb_tech_mac_10g_simulation_end
  GENERIC MAP (
    g_tb_end            => g_tb_end,
    g_nof_clk_to_rx_end => 1000
  )
  PORT MAP (
    clk       => dp_clk,
    tx_end    => tx_end,
    rx_end    => rx_end,
    tb_end    => tb_end
  );
  
END tb;
